import os
import sys

from assertpy import assert_that

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '../rabbitmqbaselibrary')))

from __version__ import VERSION  # noqa: E402


def test_version() -> None:
    assert_that(VERSION).is_equal_to('0.8.6')



from typing import List

import requests
from pyramda import map

from rabbitmqbaselibrary.common.handlers import handle_rest_response, handle_rest_response_with_body


class Binding(object):

    def __init__(self, obj: dict):
        self.source = obj.get('source')
        self.destination = obj.get('destination')
        self.destination_type = obj.get('destination_type')
        self.routing_key = obj.get('routing_key')
        self.arguments = obj.get('arguments')
        self.properties_key = obj.get('properties_key')

    def to_dict(self) -> dict:
        return {
            'source': self.source,
            'destination': self.destination,
            'destination_type': self.destination_type,
            'routing_key': self.routing_key,
            'arguments': self.arguments
        }

    def body(self) -> dict:
        return {
            'routing_key': self.routing_key,
            'arguments': self.arguments
        }

    def path(self) -> str:
        binding_type = 'e' if self.destination_type == 'exchange' else 'q'
        return 'e/{}/{}/{}'.format(self.source, binding_type, self.destination)

    def equals(self, other: object) -> bool:
        if not isinstance(other, Binding):
            return NotImplemented
        return self.source == other.source and self.destination == other.destination \
               and self.routing_key == other.routing_key and self.destination_type == other.destination_type \
               and self.arguments == other.arguments  # noqa: E127


def create_binding(broker: dict, vhost: str, binding: Binding) -> None:
    url = 'https://{}/api/bindings/{}/{}'.format(broker['host'], vhost, binding.path())
    body = binding.body()
    response = requests.post(url=url, auth=(broker['user'], broker['passwd']), json=body)
    handle_rest_response_with_body(response=response, url=url, body=body)


def delete_binding(broker: dict, vhost: str, binding: Binding) -> None:
    url = 'https://{}/api/bindings/{}/{}/{}'.format(broker['host'],
                                                    vhost, binding.path(), binding.properties_key)
    response = requests.delete(url=url, auth=(broker['user'], broker['passwd']))
    handle_rest_response(response=response, url=url)


def get_bindings_from_source(broker: dict, vhost: str, source: str) -> List[Binding]:
    url = 'https://{}/api/exchanges/{}/{}/bindings/source'.format(broker['host'], vhost, source)
    response = requests.get(url=url, auth=(broker['user'], broker['passwd']))
    handle_rest_response(response=response, url=url)
    return map(lambda i: Binding(i), response.json())


# noinspection PyDeepBugsSwappedArgs
def get_bindings(broker: dict, vhost: str) -> List[Binding]:
    url = 'https://{}/api/bindings/{}'.format(broker['host'], vhost)
    response = requests.get(url=url, auth=(broker['user'], broker['passwd']))
    handle_rest_response(response=response, url=url)
    return map(lambda i: Binding(i), response.json())



import configparser
import os


class Config(object):

    def __init__(self, path_file: str):
        self.path_file = path_file
        self.config = configparser.ConfigParser()
        self.config.read(path_file)

    def get_value(self, section: str, key: str) -> str:
        return self.config[section][key]

    def get_path(self, key: str) -> str:
        return os.path.abspath(os.path.join(os.path.dirname(self.path_file), self.config['Paths'][key]))

    def get_file_path(self, key: str, file_name: str) -> str:
        return os.path.join(self.get_path(key=key), file_name)



import json

import rabbitpy


class AmqpSenderConsumer(object):

    def __init__(self, conn_string: str):
        self.__amqp = rabbitpy.AMQP(rabbitpy.Connection(conn_string).channel())

    def publish_message(self, exchange: str, routing_key: str, message: dict) -> None:
        self.__amqp.basic_publish(exchange=exchange, routing_key=routing_key, body=json.dumps(message), mandatory=True)

    def receive_message(self, queue: str) -> dict:
        received: dict = {}
        for message in self.__amqp.basic_consume(queue=queue):
            message.ack()
            self.__amqp.basic_cancel()
            received = message.json()
            break
        return received


def build_conn_string(host: str, vhost: str, name: str, passwd: str) -> str:
    return 'amqps://{}:{}@{}/{}'.format(name, passwd, host, vhost)



import requests
from pyramda import map, contains

from rabbitmqbaselibrary.common.handlers import handle_rest_response, handle_rest_response_with_body


def is_present(broker: dict, vhost: str, name: str) -> bool:
    return contains(name, get_policies(broker=broker, vhost=vhost))


# noinspection PyDeepBugsSwappedArgs
def get_policies(broker: dict, vhost: str) -> dict:
    url = 'https://{}/api/policies/{}'.format(broker['host'], vhost)
    response = requests.get(url=url, auth=(broker['user'], broker['passwd']))
    handle_rest_response(response=response, url=url)
    return map(lambda i: i['name'], response.json())


# noinspection PyDeepBugsSwappedArgs
def create_policy(broker: dict, vhost: str, name: str, policy: dict) -> None:
    url = 'https://{}/api/policies/{}/{}'.format(broker['host'], vhost, name)
    response = requests.put(url=url, auth=(broker['user'], broker['passwd']), json=policy)
    handle_rest_response_with_body(response=response, body=policy, url=url)


def delete_policy(broker: dict, vhost: str, name: str) -> None:
    url = 'https://{}/api/policies/{}/{}'.format(broker['host'], vhost, name)
    response = requests.delete(url=url, auth=(broker['user'], broker['passwd']))
    handle_rest_response(response=response, url=url)



import json
import os

basename = os.path.dirname(os.path.realpath(__file__))


class Environments(object):
    __content: dict = {}

    def __init__(self, path_file: str):
        self.__path_file = path_file
        self.__parse()

    def __parse(self) -> None:
        with open(self.__path_file) as json_file:
            self.__content = json.load(json_file)

    def get_env(self, env: str) -> dict:
        return self.__content[env]

    def get_host(self, env: str) -> str:
        return self.__content[env]['host']



import base64
import hashlib
import os
import random
from typing import Any

import requests
from pyramda import map, keys

from rabbitmqbaselibrary.common.exceptions import NotValidPermissions, NotFoundException
from rabbitmqbaselibrary.common.handlers import handle_rest_response_with_body, handle_rest_response

AUTOGENERATED = True

# noinspection SpellCheckingInspection
pass_chars = "abcdefghijklmnopqrstuvwxyz01234567890ABCDEFGHIJKLMNOPQRSTUVWXYZ!;#$%^&*-+<>|?"
salt = os.urandom(4)


def autogenerate_password() -> str:
    return "".join(random.sample(pass_chars, 10))


# noinspection InsecureHash
def salt_hashing_passwd(passwd: str) -> str:
    tmp0 = salt + passwd.encode('utf-8')
    tmp1 = hashlib.sha256(tmp0).digest()
    return str(base64.b64encode(salt + tmp1).decode(encoding='UTF-8'))


def is_present(broker: dict, name: str) -> bool:
    try:
        result = get_user_by_name(broker=broker, name=name)
        return True if result.get('name') == name else False
    except NotFoundException:
        return False


def get_user_by_name(broker: dict, name: str) -> dict:
    url = 'https://{}/api/users/{}'.format(broker['host'], name)
    response = requests.get(url=url, auth=(broker['user'], broker['passwd']))
    handle_rest_response(response=response, url=url)
    return response.json()


def get_users(broker: dict) -> dict:
    url = 'https://{}/api/users'.format(broker['host'])
    response = requests.get(url=url, auth=(broker['user'], broker['passwd']))
    handle_rest_response(response=response, url=url)
    return map(lambda i: i['name'], response.json())


def create_user(broker: dict, name: str, pass_flag: bool, **kwargs: Any) -> str:
    passwd: str = autogenerate_password() if pass_flag is True else str(kwargs.get('password'))
    hashed_passwd = salt_hashing_passwd(passwd=passwd)
    url = 'https://{}/api/users/{}'.format(broker['host'], name)
    tags = '' if kwargs.get('tags') is None else kwargs.get('tags')
    body = {'password_hash': hashed_passwd, 'tags': tags}
    response = requests.put(url=url, auth=(broker['user'], broker['passwd']), json=body)
    handle_rest_response_with_body(response=response, url=url, body=body)
    return passwd


def delete_user(broker: dict, name: str) -> None:
    url = 'https://{}/api/users/{}'.format(broker['host'], name)
    response = requests.delete(url=url, auth=(broker['user'], broker['passwd']))
    handle_rest_response(response=response, url=url)


def add_permissions(broker: dict, vhost: str, user: str, permissions: dict) -> None:
    kk = keys(permissions)
    if not ('write' in kk and 'read' in kk and 'configure' in kk):
        raise NotValidPermissions('permissions are not complete', permissions)
    url = 'https://{}/api/permissions/{}/{}'.format(broker['host'], vhost, user)
    response = requests.put(url=url, auth=(broker['user'], broker['passwd']), json=permissions)
    handle_rest_response_with_body(response=response, url=url, body=permissions)


def delete_permissions(broker: dict, vhost: str, user: str) -> None:
    url = 'https://{}/api/permissions/{}/{}'.format(broker['host'], vhost, user)
    response = requests.delete(url=url, auth=(broker['user'], broker['passwd']))
    handle_rest_response(response=response, url=url)


def get_permissions(broker: dict, vhost: str, user: str) -> dict:
    url = 'https://{}/api/permissions/{}/{}'.format(broker['host'], vhost, user)
    response = requests.get(url=url, auth=(broker['user'], broker['passwd']))
    handle_rest_response(response=response, url=url)
    return response.json()



# noinspection PyDeepBugsSwappedArgs
import requests

from rabbitmqbaselibrary.common.handlers import handle_rest_response, handle_rest_response_with_body


# noinspection PyDeepBugsSwappedArgs
def get_definitions(broker: dict, vhost: str) -> dict:
    url = 'https://{}/api/definitions/{}'.format(broker['host'], vhost)
    response = requests.get(url=url, auth=(broker['user'], broker['passwd']))
    handle_rest_response(response, url)
    return response.json()


# noinspection PyDeepBugsSwappedArgs
def load_definitions(broker: dict, vhost: str, definitions: dict) -> None:
    url = 'https://{}/api/definitions/{}'.format(broker['host'], vhost)
    response = requests.post(url=url, auth=(broker['user'], broker['passwd']), json=definitions)
    handle_rest_response_with_body(response, url, definitions)



import json
import os

from rabbitmqbaselibrary.common.exceptions import TemplateException

basename = os.path.abspath(os.path.join(os.path.dirname(__file__), '../resources/templates'))


class Template(object):
    # noinspection PyShadowingNames
    def __init__(self, basename: str):
        self.__basename: str = basename

    # noinspection PyDeepBugsSwappedArgs
    def load_template(self, name: str) -> dict:
        try:
            with open('{}/{}.json'.format(self.__basename, name)) as json_file:
                return json.load(json_file)
        except FileNotFoundError as e:
            raise e
        except Exception as e:
            raise TemplateException(message='exception {} in template {}'.format(e.args, name),
                                    template_name=name)

    def load_template_with_args(self, name: str, args: dict) -> dict:
        try:
            with open('{}/{}.json'.format(self.__basename, name), 'r') as file:
                template = file.read()
            # noinspection StrFormat
            filed = template.format(**args)
            return json.loads(filed)
        except FileNotFoundError as e:
            raise e
        except Exception as e:
            # noinspection PyDeepBugsSwappedArgs
            raise TemplateException(message='exception {} in template {}'.format(e.args, name),
                                    template_name=name)



import json
import uuid
from datetime import datetime
from typing import List, Dict


class Report(object):
    __buffer: List[Dict] = []
    __context: dict = {}

    def set_context(self, context: dict) -> None:
        self.__context = context

    def append_event(self, name: str, record: dict) -> None:
        record.update({'context': self.__context})
        record.update({
            'id': str(uuid.uuid4()),
            'name': name,
            'timestamp': datetime.now().isoformat()
        })
        self.__buffer.append(record)

    def report(self) -> str:
        return json.dumps(self.__buffer, indent=4)



import requests
from pyramda import map, contains

from rabbitmqbaselibrary.common.exceptions import ServerErrorException, VhostNotFound, VhostAlreadyExists
from rabbitmqbaselibrary.common.handlers import handle_rest_response


def is_present(broker: dict, vhost: str) -> bool:
    return contains(vhost, get_vhosts(broker=broker))


def get_vhosts(broker: dict) -> dict:
    url = 'https://{}/api/vhosts'.format(broker['host'])
    response: requests.Response = requests.get(url=url, auth=(broker['user'], broker['passwd']))
    if not response.ok:
        # noinspection PyTypeChecker
        raise ServerErrorException(str(response.status_code), url=url)
    return map(lambda i: i['name'], response.json())


def create_vhost(broker: dict, vhost: str) -> None:
    if is_present(broker=broker, vhost=vhost):
        raise VhostAlreadyExists(vhost)
    url = 'https://{}/api/vhosts/{}'.format(broker['host'], vhost)
    response = requests.put('https://{}/api/vhosts/{}'.format(broker['host'], vhost),
                            auth=(broker['user'], broker['passwd']))
    handle_rest_response(response=response, url=url)


def delete_vhost(broker: dict, vhost: str) -> None:
    if not is_present(broker=broker, vhost=vhost):
        raise VhostNotFound(vhost)
    url = 'https://{}/api/vhosts/{}'.format(broker['host'], vhost)
    response = requests.delete(url, auth=(broker['user'], broker['passwd']))
    handle_rest_response(response=response, url=url)



from setuptools import setup

setup(
    author="David Navarro Alvarez",
    author_email="me@davengeo.com",
    description="rabbitmq-base-library dist",
    url="https://github.com/davengeo/rabbitme-base-library",
    name="rabbitmq-base-library",
    packages=['rabbitmqbaselibrary', 'rabbitmqbaselibrary.bindings', 'rabbitmqbaselibrary.common',
              'rabbitmqbaselibrary.definitions', 'rabbitmqbaselibrary.exchanges', 'rabbitmqbaselibrary.messages',
              'rabbitmqbaselibrary.policies', 'rabbitmqbaselibrary.queues', 'rabbitmqbaselibrary.users',
              'rabbitmqbaselibrary.vhost'],
    install_requires=['requests', 'argparse', 'pyramda', 'rabbitpy'],
)



import requests

from rabbitmqbaselibrary.common.exceptions import BadRequest, ServerErrorException, Unauthorised, NotFoundException


# noinspection PyPep8Naming
def handle_rest_response_with_body(response: requests.Response, url: str, body: object) -> None:
    if not response.ok:
        if response.status_code == 404:
            raise NotFoundException(message='resource not found', url=url)
        if response.status_code == 400:
            raise BadRequest(message='bad request', url=url, body=body)
        if response.status_code == 500:
            raise ServerErrorException(message='server exception', url=url)
        if response.status_code == 401:
            raise Unauthorised(url=url)
        else:
            raise Exception(response.status_code, response.reason)


# noinspection PyPep8Naming
def handle_rest_response(response: requests.Response, url: str) -> None:
    if not response.ok:
        if response.status_code == 404:
            raise NotFoundException(message='resource not found', url=url)
        if response.status_code == 500:
            raise ServerErrorException(message='server exception', url=url)
        if response.status_code == 401:
            raise Unauthorised(url=url)
        else:
            raise Exception(response.status_code, response.reason)



class VhostAlreadyExists(Exception):

    def __init__(self, vhost: str):
        self.vhost = vhost


class VhostNotFound(Exception):

    def __init__(self, vhost: str):
        self.vhost = vhost


class Unauthorised(Exception):

    def __init__(self, url: str):
        self.url = url


class NotFoundException(Exception):

    def __init__(self, message: str, url: str):
        self.message = message
        self.url = url


class ServerErrorException(Exception):

    def __init__(self, message: str, url: str):
        self.message = message
        self.url = url


class BadRequest(Exception):

    def __init__(self, message: str, url: str, body: object):
        self.message = message
        self.url = url
        self.body = body


class TemplateException(BaseException):
    def __init__(self, message: str, template_name: str):
        self.message = message
        self.template_name = template_name


class NotValidPermissions(BaseException):
    def __init__(self, message: str, permissions: dict):
        self.message = message
        self.permissions = permissions
